/*
This is based on my simpler webcam viewer.
Written 30/05/13

Commands:

Hold Ctrl to increase movement by 10x

Left arrow    Move crosshairs left
Right arrow    Move crosshairs right
Up arrow    Move crosshairs up
Down arrow    Move crosshairs down

Shift + left arrow  Decrease width
Shift + right arrow  Increase width
Shift + up arrow  Increase height
Shift + down arrow  Decrease height

Page up      Increase line thickness
Page down    Decrease line thickness

Shift + page up    Rotate anticlockwise
Shift + page down  Rotate clockwise

Select type:
1      Plain crosshairs
2      Plain crosshairs with space
3      Two pairs of parallel lines
4      Two pairs of parallel lines with space (corner marks)
5      Rectangle
6      Rectangle + plain crosshairs
7      Circle
8      Circle + plain crosshairs
9      N/A
0      No crosshairs

C    Cycle through colours - black (0), white (1), red (2), green (3), blue (4), cyan (5), magenta (6), yellow (7)
R    Reset parameters
X    Exit program
Mouse click/drag - reset origin
*/
import processing.video.*; // import video processing stuff
import java.awt.*; // import Java components (only Choice is used)
PFont f; // font primitive
Choice guiChoice = new Choice(); // create new drop-down list
int programstatus; // current "mode" the program is in. 0 is initial run, anything else is, duh, otherwise!
String[] cameras; // string array containing details of each available camera/video device
Capture cam; // capture device

// Declarations for crosshair parameters
int ch_x; // x position of center rel to center of screen
int ch_y; // y position of center rel to center of screen
int ch_w; // width
int ch_h; // height
int ch_t; // thickness
int ch_a; // angle. This is divided by 5 to get the ACTUAL angle, so we have a resolution of 0.2°
int ch_type; // crosshair type
int ch_color; // color index to use
color[] colors=new color[8]; // array to store colors

float x1,y1,x2,y2; // temporary variables used for calculations

int x_origin; // actual X and Y pixel origins of crosshairs
int y_origin;
int maxlength; // maximum length of line which fits diagonally on screen

boolean shiftflag; // flag which determines whether shift is held down or not
boolean ctrlflag; // flag which determines whether ctrl is held down or not

void setup() {
  size(700,370); // set initial frame size. This just has to fit the drop-down box and notification text
  if(frame != null) 
  { 
    frame.setResizable(true); // set resizable (so can make it fit the webcam feed later)
  }
  
  // Initialise color array
  colors[0]=color(0,0,0); // black
  colors[1]=color(255,255,255); // white
  colors[2]=color(255,0,0); // red
  colors[3]=color(0,255,0); // green
  colors[4]=color(0,0,255); // blue
  colors[5]=color(0,255,255); // cyan
  colors[6]=color(255,0,255); // magenta
  colors[7]=color(255,255,0); // yellow
  
  f=loadFont("CenturyGothic-Bold-15.vlw"); // load font
  textFont(f);
  fill(0); // set color to black
  text("Choose camera from the list and then click anywhere in window to continue",10,50); // display text to user
  text("Once video is displayed, click inside window and you can then use keyboard commands:",10,80);
  text("Up/down/left/right arrow keys: move crosshairs",10,100);
  text("Shift+left/right arrows: increase/decrease width",10,120);
  text("Shift+up/down arrows: increase/decrease height",10,140);
  text("Page up/down: increase/decrease line thickness",10,160);
  text("Shift+page up/down: rotate ACW/CW",10,180);
  text("Main keyboard 1,2,3,4,5,6,7,8,0: select crosshair type",10,200);
  text("C: cycle through colours",10,220);
  text("R: reset all parameters",10,240);
  text("X: exit program",10,260);
  text("Hold ctrl with other keys to increase by 10x",10,280);
  text("Ctrl + click & drag with mouse to set/move the origin",10,300);
  text("Note that some commands only apply to certain crosshair types",10,330);
  fill(color(255,0,0));
  text("Lindsay Wilson 29/05/13.  www.imajeenyus.com  lindsay@imajeenyus.com",10,360);
  
  programstatus=0; // initialise program status
  
  cameras=Capture.list(); // get list of available cameras
  
  if (cameras.length==0) {
    println("There are no cameras available for capture.");
    exit();
  } else {
    for (int i=0;i<cameras.length; i++) {
      guiChoice.add(cameras[i]); // populate drop-down list with all cameras
    }
    add(guiChoice); // finally, display the drop-down list.
  }
}

void draw() {
  if (programstatus==0)
  {
    // if this is the first run, then only continue if the user has selected a camera
    // and clicked somewhere on the window
    if(mousePressed)
    {
      println(guiChoice.getSelectedIndex()); // print selection
      cam=new Capture(this,cameras[guiChoice.getSelectedIndex()]); // create new camera capture using the specified camera
      remove(guiChoice); // remove drop-down list, otherwise it appears on top of the video feed
      cam.start(); // start
      programstatus=1; // set status so program can move on to displaying actual frames
      
      // Initialise all of the crosshair parameters at this stage.
      ch_x=0;
      ch_y=0;
      ch_w=10;
      ch_h=10;
      ch_t=2;
      ch_a=0;
      ch_type=1;
      ch_color=0;
      
      // Initialise shift and ctrl flags
      shiftflag=false;
      ctrlflag=false;
      
      // Load main font for use in display
      f=loadFont("Consolas-Bold-20.vlw"); // load font
      textFont(f);
    }
  } else   
  {
    if (cam.available() == true) // only do stuff if there's a frame available from the webcam 
    { 
      cam.read(); // get frame from camera and store in cam
      frame.setSize(cam.width,cam.height); // resize frame to match camera image
      image(cam,0,0); // finally, display the camera image
      
      // Calculate actual pixel origins
      // Remember we have to subtract Y from cam.height when actually drawing
      x_origin=(cam.width/2)+ch_x;
      y_origin=(cam.height/2)+ch_y;
      
      // Calculate maximum line length (actually just the diagonal length of window)
      maxlength=int(sqrt((pow(float(cam.width),2))+(pow(float(cam.height),2))));
      
      // Now draw crosshairs
      strokeWeight(ch_t); // set stroke weight
      stroke(colors[ch_color]); // set color
      switch(ch_type)
      {
        case 1:
          // Plain crosshairs
          line (x_origin-(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin-(maxlength*sin(ch_a*PI/900))),x_origin+(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin+(maxlength*sin(ch_a*PI/900))));
          line (x_origin-(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin+(maxlength*cos(ch_a*PI/900))),x_origin+(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin-(maxlength*cos(ch_a*PI/900))));
          break;
        case 2:
          // Plain crosshairs with space
          line (x_origin-(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin-(maxlength*sin(ch_a*PI/900))),x_origin-((ch_w/2)*cos(ch_a*PI/900)),cam.height-(y_origin-((ch_w/2)*sin(ch_a*PI/900))));
          line (x_origin+((ch_w/2)*cos(ch_a*PI/900)),cam.height-(y_origin+((ch_w/2)*sin(ch_a*PI/900))),x_origin+(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin+(maxlength*sin(ch_a*PI/900))));
          line (x_origin-(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin+(maxlength*cos(ch_a*PI/900))),x_origin-((ch_h/2)*sin(ch_a*PI/900)),cam.height-(y_origin+((ch_h/2)*cos(ch_a*PI/900))));
          line (x_origin+((ch_h/2)*sin(ch_a*PI/900)),cam.height-(y_origin-((ch_h/2)*cos(ch_a*PI/900))),x_origin+(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin-(maxlength*cos(ch_a*PI/900))));         
          break;
        case 3:
          // Two pairs of parallel lines
          x1 = x_origin-((ch_w/2)*cos(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          y1 = y_origin-((ch_w/2)*sin(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          x2 = x_origin-((ch_w/2)*cos(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          y2 = y_origin-((ch_w/2)*sin(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1 = x_origin+((ch_w/2)*cos(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          y1 = y_origin+((ch_w/2)*sin(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          x2 = x_origin+((ch_w/2)*cos(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          y2 = y_origin+((ch_w/2)*sin(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1 = x_origin-((ch_h/2)*sin(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          y1 = y_origin+((ch_h/2)*cos(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          x2 = x_origin-((ch_h/2)*sin(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          y2 = y_origin+((ch_h/2)*cos(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1 = x_origin+((ch_h/2)*sin(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          y1 = y_origin-((ch_h/2)*cos(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          x2 = x_origin+((ch_h/2)*sin(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          y2 = y_origin-((ch_h/2)*cos(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          break;
        case 4:
          // Two pairs of parallel lines with space (corner marks)
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900))+(maxlength*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900))+(maxlength*sin(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900))-(maxlength*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900))-(maxlength*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          break;
        case 5:
          // Rectangle
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          break;
        case 6:
          // Rectangle + plain crosshairs
          line (x_origin-(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin-(maxlength*sin(ch_a*PI/900))),x_origin+(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin+(maxlength*sin(ch_a*PI/900))));
          line (x_origin-(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin+(maxlength*cos(ch_a*PI/900))),x_origin+(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin-(maxlength*cos(ch_a*PI/900))));
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin-((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin+((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))+((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))+((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          x1=x_origin+((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y1=y_origin-((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          x2=x_origin-((ch_h/2)*sin(ch_a*PI/900))-((ch_w/2)*cos(ch_a*PI/900));
          y2=y_origin+((ch_h/2)*cos(ch_a*PI/900))-((ch_w/2)*sin(ch_a*PI/900));
          line(x1,cam.height-y1,x2,cam.height-y2);
          break;
        case 7:
          // Circle
          noFill();
          ellipseMode(CENTER);
          ellipse(x_origin,cam.height-y_origin,ch_w,ch_w);
          break;
        case 8:
          // Circle + plain crosshairs
          line (x_origin-(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin-(maxlength*sin(ch_a*PI/900))),x_origin+(maxlength*cos(ch_a*PI/900)),cam.height-(y_origin+(maxlength*sin(ch_a*PI/900))));
          line (x_origin-(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin+(maxlength*cos(ch_a*PI/900))),x_origin+(maxlength*sin(ch_a*PI/900)),cam.height-(y_origin-(maxlength*cos(ch_a*PI/900))));
          noFill();
          ellipseMode(CENTER);
          ellipse(x_origin,cam.height-y_origin,ch_w,ch_w);
          break;
        case 0:
          // No crosshairs
          break;
      }
      
      // Finally, show current parameters if crosshair type <>0
      if(ch_type!=0) {
      fill(colors[ch_color]);
      text("X",10,20);
      text(ch_x,25,20);
      text("Y",10,40);
      text(ch_y,25,40);
      text("W",10,60);
      text(ch_w,25,60);
      text("H",10,80);
      text(ch_h,25,80);
      text("A",10,100);
      text(float(ch_a)/5,25,100);
      }
    }
   }
}

void keyPressed()
{
  // Handle things when key is pressed
  println(keyCode);
  if(keyCode==16) {shiftflag=true;} // set shift flag
  if(keyCode==17) {ctrlflag=true;} // set ctrl flag
  switch(keyCode)
  {
    case 37: // left arrow
      if(shiftflag)
      {
        if(ctrlflag==true)
        {
          if(ch_w>9) {ch_w-=10;}
        } else {
          if(ch_w>0) {ch_w--;}
        }
      } else {
        if(ctrlflag==true) {
          ch_x-=10;
        } else {
          ch_x--;
        }
      }
      break;
    case 39: // right arrow
      if(shiftflag)
      {
        if(ctrlflag==true) {
          ch_w+=10;
        } else {
          ch_w++;
        }
      } else
      {
        if(ctrlflag==true) {
          ch_x+=10;
        } else {
          ch_x++;
        }
      }
      break;
    case 38: // up arrow
      if(shiftflag)
      {
        if(ctrlflag==true) {
          ch_h+=10;
        } else {
          ch_h++;
        }
      } else
      {
        if(ctrlflag==true) {
          ch_y+=10;
        } else {
          ch_y++;
        }
      }
      break;
    case 40: // down arrow
      if(shiftflag)
      {
        if(ctrlflag==true) {
          if(ch_h>9) {ch_h-=10;}
        } else {
          if(ch_h>0) {ch_h--;}
        }
      } else
      {
        if(ctrlflag==true) {
          ch_y-=10;
        } else {
          ch_y--;
        }
      }
      break;
    case 33: // page up
      if(shiftflag)
      {
        if(ctrlflag==true) {
          ch_a+=10;
        } else {
          ch_a++;
        }
      } else
      {
        if(ctrlflag==true) {
          ch_t+=10;
        } else {
          ch_t++;
        }
      }
      break; 
    case 34: // page down
      if(shiftflag)
      {
        if(ctrlflag==true) {
          ch_a-=10;
        } else {
          ch_a--;
        }
      } else
      {
        if(ctrlflag==true) {
          if(ch_t>9) {ch_t-=10;}
        } else {
          if(ch_t>0) {ch_t--;}
        }
      }
      break;
    case 49: // type 1
      ch_type=1;
      break;
    case 50: // type 2
      ch_type=2;
      break;
    case 51: // type 3
      ch_type=3;
      break;
    case 52: // type 4
      ch_type=4;
      break;
    case 53: // type 5
      ch_type=5;
      break;
    case 54: // type 6
      ch_type=6;
      break;
    case 55: // type 7
      ch_type=7;
      break;
    case 56: // type 8
      ch_type=8;
      break;
    case 57: // type 9
      ch_type=9;
      break;
    case 48: // type 0 (none)
      ch_type=0;
      break;
    case 67: // C - cycle colors
      if(ch_color>6) {ch_color=0;} else {ch_color++;}
      break;
    case 82: // R - reset everything
      ch_x=0;
      ch_y=0;
      ch_w=10;
      ch_h=10;
      ch_t=2;
      ch_a=0;
      ch_type=1;
      ch_color=0;
      break;
    case 88: // X - exit
      exit();
      break;
  } 
}

void keyReleased()
{
  if(keyCode==16) {shiftflag=false;} // clear shift flag
  if(keyCode==17) {ctrlflag=false;} // clear ctrl flag
}

void mousePressed()
{
  // Update origin with mouse position, but only if we are actually viewing the camera (i.e. don't respond
  // to any mouse clicks on the splash screen)
  if(programstatus!=0 && ctrlflag==true) {
    ch_x=mouseX-(cam.width/2);
    ch_y=(cam.height/2)-mouseY;
  }
}

void mouseDragged()
{
  // Update origin with mouse position, but only if we are actually viewing the camera (i.e. don't respond
  // to any mouse clicks on the splash screen)
  if(programstatus!=0 && ctrlflag==true) {
    ch_x=mouseX-(cam.width/2);
    ch_y=(cam.height/2)-mouseY;
  }
}
